/************************************************************************
 *   IRC - Internet Relay Chat, m_svscmds.c
 *   Copyright (C) 2003 PleXusIRCd Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_svsalias.c,v 1.1.1.1 2004/10/17 11:18:26 jon Exp $
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "channel.h"
#include "channel_mode.h"
#include "common.h"		/* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "s_user.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "hash.h"
#include "whowas.h"

/* Declare the void's initially up here, as modules dont have an
 * include file, we will normally have client_p, source_p, parc
 * and parv[] where:
 *
 * client_p == client issuing command
 * source_p == where the command came from
 * parc     == the number of parameters
 * parv     == an array of the parameters
 */
static void ms_ns(struct Client *client_p, struct Client *source_p, int parc, char *parv[]);

static void ms_cs(struct Client *client_p, struct Client *source_p, int parc, char *parv[]);

static void ms_os(struct Client *client_p, struct Client *source_p, int parc, char *parv[]);

static void ms_ms(struct Client *client_p, struct Client *source_p, int parc, char *parv[]);

static void ms_bs(struct Client *client_p, struct Client *source_p, int parc, char *parv[]);

static void ms_hs(struct Client *client_p, struct Client *source_p, int parc, char *parv[]);

static void ms_ident(struct Client *client_p, struct Client *source_p, int parc, char *parv[]);

static void ms_vhost(struct Client *client_p, struct Client *source_p, int parc, char *parv[]);

#define MAXLINE  512

/* Show the commands this module can handle in a msgtab
 * and give the msgtab a name, here its test_msgtab
 */
struct Message ns_msgtab = {
	"NS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_ns, m_error, ms_ns}
};
struct Message nickserv_msgtab = {
	"NICKSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_ns, m_error, ms_ns}
};
struct Message cs_msgtab = {
	"CS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_cs, m_error, ms_cs}
};
struct Message chanserv_msgtab = {
	"CHANSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_cs, m_error, ms_cs}
};
struct Message os_msgtab = {
	"OS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_os, m_error, ms_os}
};
struct Message operserv_msgtab = {
	"OPERSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_os, m_error, ms_os}
};
struct Message ms_msgtab = {
	"MS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_ms, m_error, ms_ms}
};
struct Message memoserv_msgtab = {
	"MEMOSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_ms, m_error, ms_ms}
};
struct Message bs_msgtab = {
	"BS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_bs, m_error, ms_bs}
};
struct Message botserv_msgtab = {
	"BOTSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_bs, m_error, ms_bs}
};
struct Message hs_msgtab = {
	"HS", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_hs, m_error, ms_hs}
};
struct Message helpserv_msgtab = {
	"HELPSERV", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_error, ms_hs, m_error, ms_hs}
};
struct Message ident_msgtab = {
	"IDENTIFY", 0, 0, 1, 2, MFLG_SLOW, 0,
	{m_error, ms_ident, m_error, ms_ident}
};
struct Message vhost_msgtab = {
	"VHOST", 0, 0, 1, 3, MFLG_SLOW, 0,
	{m_error, ms_vhost, m_error, ms_vhost}
};

/* Thats the msgtab finished */

#ifndef STATIC_MODULES
/* Here we tell it what to do when the module is loaded */
void
_modinit(void)
{
	/*
	 * This will add the commands in test_msgtab (which is above) 
	 */
	mod_add_cmd(&ns_msgtab);
	mod_add_cmd(&nickserv_msgtab);
	mod_add_cmd(&cs_msgtab);
	mod_add_cmd(&chanserv_msgtab);
	mod_add_cmd(&os_msgtab);
	mod_add_cmd(&operserv_msgtab);
	mod_add_cmd(&ms_msgtab);
	mod_add_cmd(&memoserv_msgtab);
	mod_add_cmd(&bs_msgtab);
	mod_add_cmd(&botserv_msgtab);
	mod_add_cmd(&hs_msgtab);
	mod_add_cmd(&helpserv_msgtab);
	mod_add_cmd(&ident_msgtab);
	mod_add_cmd(&vhost_msgtab);
}

/* here we tell it what to do when the module is unloaded */
void
_moddeinit(void)
{
	/*
	 * This will remove the commands in test_msgtab (which is above) 
	 */
	mod_del_cmd(&ns_msgtab);
	mod_del_cmd(&nickserv_msgtab);
	mod_del_cmd(&cs_msgtab);
	mod_del_cmd(&chanserv_msgtab);
	mod_del_cmd(&os_msgtab);
	mod_del_cmd(&operserv_msgtab);
	mod_del_cmd(&ms_msgtab);
	mod_del_cmd(&memoserv_msgtab);
	mod_del_cmd(&bs_msgtab);
	mod_del_cmd(&botserv_msgtab);
	mod_del_cmd(&hs_msgtab);
	mod_del_cmd(&helpserv_msgtab);
	mod_del_cmd(&ident_msgtab);
	mod_del_cmd(&vhost_msgtab);
}

/* When we last modified the file (shown in /modlist), this is usually:
 */
const char *_version = "$Revision: 1.1.1.1 $";
#endif

char *
getString(int parc, char *parv[])
{
	char *final;
	char temp[MAXLINE];
	int ii = 0;

	final = MyMalloc(sizeof(char));
	*final = '\0';

	while (ii < parc) {
		ircsprintf(temp, "%s%s", parv[ii], ((ii + 1) >= parc) ? "" : " ");
		final = MyRealloc(final, strlen(final) + strlen(temp) + sizeof(char));
		strcat(final, temp);
		++ii;
	}

	return (final);
}				/* getString() */

static void
ms_ident(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;

	if(parc < 2) {
		sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "MODE");
	}

	if(target_p = find_server(SERVICES)) {
		sendto_one(target_p, ":%s PRIVMSG %s@%s :IDENTIFY %s", parv[0], "NICKSERV",
		    SERVICES, parv[1]);
	} else {
		sendto_one(source_p,
		    ":%s 425 %s :Services are currently down, if you have any questions please join #help.",
		    me.name, parv[0]);
	}
	return;
}

static void
ms_vhost(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;

	if(parc < 2) {
		sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "MODE");
	}

	if(target_p = find_server(HOSTSERV)) {
		sendto_one(target_p, ":%s PRIVMSG %s :VHOST %s %s", parv[0], "HOSTSERV", parv[1],
		    parv[2]);
	} else {
		sendto_one(source_p,
		    ":%s 425 %s :Services are currently down, if you have any questions please join #help.",
		    me.name, parv[0]);
	}
	return;
}


static void
ms_ns(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	char *servst;

	if(parc < 2) {
		sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "MODE");
	}

	if(target_p = find_server(SERVICES)) {
		servst = getString(parc - 1, parv + 1);

		sendto_one(target_p, ":%s PRIVMSG %s@%s :%s", parv[0], "NICKSERV", SERVICES,
		    servst);
	} else {
		sendto_one(source_p,
		    ":%s 425 %s :Services are currently down, if you have any questions please join #help.",
		    me.name, parv[0]);
	}
	return;
}

static void
ms_cs(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	char *servst;

	if(parc < 2) {
		sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "MODE");
	}

	if(target_p = find_server(SERVICES)) {
		servst = getString(parc - 1, parv + 1);

		sendto_one(target_p, ":%s PRIVMSG %s@%s :%s", parv[0], "CHANSERV", SERVICES,
		    servst);
	} else {
		sendto_one(source_p,
		    ":%s 425 %s :Services are currently down, if you have any questions please join #help.",
		    me.name, parv[0]);
	}
	return;

}

static void
ms_os(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	char *servst;

	if(parc < 2) {
		sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "MODE");
	}

	if(target_p = find_server(SERVICES)) {
		servst = getString(parc - 1, parv + 1);

		sendto_one(target_p, ":%s PRIVMSG %s@%s :%s", parv[0], "OPERSERV", SERVICES,
		    servst);
	} else {
		sendto_one(source_p,
		    ":%s 425 %s :Services are currently down, if you have any questions please join #help.",
		    me.name, parv[0]);
	}
	return;

}

static void
ms_ms(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	char *servst;

	if(parc < 2) {
		sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "MODE");
	}

	if(target_p = find_server(SERVICES)) {
		servst = getString(parc - 1, parv + 1);

		sendto_one(target_p, ":%s PRIVMSG %s@%s :%s", parv[0], "MEMOSERV", SERVICES,
		    servst);
	} else {
		sendto_one(source_p,
		    ":%s 425 %s :Services are currently down, if you have any questions please join #help.",
		    me.name, parv[0]);
	}
	return;

}

static void
ms_bs(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	char *servst;

	if(parc < 2) {
		sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "MODE");
	}

	if(target_p = find_server(SERVICES)) {
		servst = getString(parc - 1, parv + 1);

		sendto_one(target_p, ":%s PRIVMSG %s@%s :%s", parv[0], "BOTSERV", SERVICES, servst);
	} else {
		sendto_one(source_p,
		    ":%s 425 %s :Services are currently down, if you have any questions please join #help.",
		    me.name, parv[0]);
	}
	return;

}

static void
ms_hs(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	char *servst;

	if(parc < 2) {
		sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "MODE");
	}

	if(target_p = find_server(SERVICES)) {
		servst = getString(parc - 1, parv + 1);

		sendto_one(target_p, ":%s PRIVMSG %s@%s :%s", parv[0], "HELPSERV", SERVICES,
		    servst);
	} else {
		sendto_one(source_p,
		    ":%s 425 %s :Services are currently down, if you have any questions please join #help.",
		    me.name, parv[0]);
	}
	return;

}
